<?php

namespace SM\Service;

use SM\Factory\ApropriacaoRecursosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class ApropriacaoRecursosService
{
    protected $entityManager;
    protected $apropriacaoRecursosFactory;
    protected $environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, ApropriacaoRecursosFactory $apropriacaoRecursosFactory, Environment $environment, NewCode $newCode)
    {
        $this->apropriacaoRecursosFactory   = $apropriacaoRecursosFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
    }

    public function getApropAgendamento()
    {
        return $this->apropriacaoRecursosFactory->getApropAgendamento();
    } 

    public function getRecursosApropAgendamento()
    {
        $nrorg = $this->environment->getNrorg();
        $cdtipo = '1'; // Apenas Recursos do tipo 'Pessoa'

        $parametros = array(
            "NRORG"     => $nrorg,
            "CDTIPO"    => $cdtipo
        );

        return $this->apropriacaoRecursosFactory->getRecursosApropAgendamento($parametros);
    }

    public function getParceiroApropAgendamento()
    {
        return $this->apropriacaoRecursosFactory->getParceiroApropAgendamento();
    }

    public function getApontamentoApropAgen($parametros)
    {
        $nrorg = $this->environment->getNrorg();

        $dataParams = array(
            "NRORG"                 => $nrorg,
            "NRAPROPAGENDAMENTO"    => isset($parametros["NRAPROPAGENDAMENTO"])? $parametros["NRAPROPAGENDAMENTO"] : null,
        );

        return $this->apropriacaoRecursosFactory->getApontamentoApropAgen($dataParams);
    }

    private function timeToDecimal($time) {
        list($hours, $minutes) = explode(':', $time);
        $decimalMinutes = $minutes / 60;
        $decimalTime = $hours + $decimalMinutes;
        return number_format($decimalTime, 2); // Formata para 2 casas decimais
    }

    public function salvarApontamento($parametros)
    {
        $nrorg = $this->environment->getNrorg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            if ($parametros["NRAPONTAMENTOAPROPAGEN"] == 'NOVO_APONTAMENTO') {
                $novoCodigo = $this->newCode->getCodeByTableName("FCT_APONTAMENTOAPROPAGEN", $nrorg, 1, 10, true);
                
                $dataParams = array(
                    "NRAPONTAMENTOAPROPAGEN"    => $novoCodigo,
                    "NRAPROPAGENDAMENTO"        => $parametros["NRAPROPAGENDAMENTO"],    
                    "IDTIPOAPONTAMENTO"         => $parametros["IDTIPOAPONTAMENTO"],     
                    "IDMOTIVOSAIDA"             => $parametros["IDMOTIVOSAIDA"],         
                    "DTHRINICIOAPONTCDA"        => $parametros["DTHRINICIOAPONTCDA"],    
                    "DTHRFIMCIOAPONTCDA"        => $parametros["DTHRFIMCIOAPONTCDA"],
                    "QTTEMPOAPONT"              => $this->timeToDecimal($parametros["QTTEMPOAPONT"]),                
                    "DSOBSAPONTAMENTO"          => $parametros["DSOBSAPONTAMENTO"],      
                    "NRORG"                     => $nrorg,                 
                    "IDATIVO"                   => 'S',                          
                    "NRORGINCLUSAO"             => $nrorg,         
                    "CDOPERINCLUSAO"            => $cdoperador,                 
                );

                $this->apropriacaoRecursosFactory->insertApontamentoApropAgen($dataParams);

            } else {
                $dataParams = array(
                    "NRAPONTAMENTOAPROPAGEN"    => $parametros["NRAPONTAMENTOAPROPAGEN"],   
                    "IDTIPOAPONTAMENTO"         => $parametros["IDTIPOAPONTAMENTO"],     
                    "IDMOTIVOSAIDA"             => $parametros["IDMOTIVOSAIDA"],         
                    "DTHRINICIOAPONTCDA"        => $parametros["DTHRINICIOAPONTCDA"],    
                    "DTHRFIMCIOAPONTCDA"        => $parametros["DTHRFIMCIOAPONTCDA"],
                    "QTTEMPOAPONT"              => $this->timeToDecimal($parametros["QTTEMPOAPONT"]),              
                    "DSOBSAPONTAMENTO"          => $parametros["DSOBSAPONTAMENTO"],      
                    "NRORG"                     => $nrorg,                 
                    "IDATIVO"                   => 'S',            
                    "NRORGULTATU"               => $nrorg,           
                    "CDOPERULTATU"              => $cdoperador,          
                );
                
                $this->apropriacaoRecursosFactory->updateApontamentoApropAgen($dataParams);
            }
        
            $this->connection->commit();
            return "OK";
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    } 

    public function deleteApontamento($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->apropriacaoRecursosFactory->deleteApontamento(FindService::formatarArray2($parametros['NRAPONTAMENTOAPROPAGEN']));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

}